#!/usr/bin/python3
# app version
__version__ = "1.0.19"  # type: str

import os
import shutil
from colorama import init
from termcolor import colored
from argparse import ArgumentParser

# use Colorama to make Termcolor work on Windows too
init()

# functions imports
from protobuf_compiler.output_folder import preparing_output
from protobuf_compiler.input_folder import define_input
from protobuf_compiler.args_validation import args_validation

# global vars
packages = [];
packages_no_added = set()
package_name = ''
package_version = ''
input_dir = ''
output_dir = ''
git_repository = ''
git_repository_token = ''

def main():
    """
    app main
    :return:
    """
    parser = ArgumentParser(
        prog="protopy.py",
        description='Compile all protobuf files and create a single package \
        distribution for can be installed with pip. app version: ' + __version__

    )
    parser.add_argument("-d", "--dir", dest="origin",
                        help="folder path where the .proto files are located, can be relative \
                        ./protos or absolute /path/to/protos", metavar="PROTO_DIR", default=os.getcwd())
    parser.add_argument("-p", "--package", dest="package", default="1.0.0",
                        help="build package name", metavar="PACKAGE_NAME")
    parser.add_argument("-o", "--output", dest="output", default=".",
                        help="output folder for save single package .tar.gz, can be relative \
                        ./dist or absolute /path/to/dist", metavar="OUTPUT_DIR")
    parser.add_argument("-g", "--git", dest="repository",
                        help="git reopsitory clone http url where the .proto files are located", metavar="URL",
                        default="")
    parser.add_argument("-t", "--token", dest="token",
                        help="github or gitlab auth api token", metavar="TOKEN",
                        default="")
    parser.add_argument("-v", "--version", dest="version", default="1.0.0",
                        help="tag version for build pacakge", metavar="VERSION")
    parser.add_argument("-env", "--envars", dest="envars", default="false",
                        help="load inputs from envars \n\
                        -d=PROTO_INPUT_DIR \
                        -o=PROTO_OUTPUT_DIR \
                        -g=PROTO_GIT_REPO_URL \
                        -t=PROTO_GIT_REPO_TOKEN \
                        -v=PROTO_PACKAGE_VERSION \
                        -p=PROTO_PACKAGE_NAME)", metavar="true")


    args = parser.parse_args()
    if args.envars is not None and str().lower() == "true":
        package_name = os.getenv("PROTO_PACKAGE_NAME", "")
        package_version = os.getenv("PROTO_PACKAGE_VERSION", "")
        git_repository = os.getenv("PROTO_GIT_REPO_URL", "")
        git_repository_token = os.getenv("PROTO_GIT_REPO_TOKEN", "")
        input_dir = define_input(os.getenv("PROTO_INPUT_DIR", ""), git_repository, git_repository_token)
        output_dir = preparing_output(os.getenv("PROTO_OUTPUT_DIR", ""), package_name)
    else:
        # define package name
        package_name = args.package
        # define package version
        package_version = args.version
        # define the input dir
        input_dir = define_input(args.origin, args.repository, args.token)
        print(colored("* input dir: ", "yellow"), input_dir)
        # define the output dir
        output_dir = preparing_output(args.output, package_name)
        print(colored("* output dir: ", "green"), output_dir)
        git_repository_token = args.token
        git_repository = args.repository

    args_validation(git_repository=git_repository,
                    proto_input_dir=input_dir,
                    proto_output_dir=output_dir,
                    proto_package_version=package_version,
                    git_repository_token=git_repository_token)

    errors = 0
    files=[]
    for r, d, f in os.walk(input_dir):
        for file in f:
            if ".proto" in file.lower():
                files.append(os.path.join(r, file))
    for f in files:
        file_name = f.split(os.path.sep)[-1]
        folder = f.replace(file_name,"").replace(input_dir + os.path.sep, "")
        command = "python3 -m grpc_tools.protoc -I. \
                                 --proto_path=" + input_dir + " \
                                 --python_out=" + os.path.join(output_dir, package_name) + " \
                                 --grpc_python_out=" + os.path.join(output_dir, package_name) + " \
                                " + f
        out = os.system(command)
        if out == 0:
            packages.append(folder.replace(os.path.sep, ".").replace("-", "_"))
            asd = os.path.join(output_dir, package_name, folder.replace("-", "_"),
                               "__init__.py")
            os.system("touch " + asd)
            # print("DONE: ", os.path.join(input_dir, folder, file))
        else:
            errors += 1
            packages_no_added.add(folder.replace("-", "_"))
    with open(os.path.join(output_dir, package_name, "setup.py"), "+w") as setup:
        setup.write("""
from setuptools import setup

setup(
    name='""" + package_name + """',
    version='""" + package_version + """',
    packages=""" + str(packages) + """,
    url='github.com/netsaj',
    license='ISC',
    author='internal',
    author_email='me@localhost',
    description=''
)
        """)
    command = "cd " + os.path.join(output_dir, package_name) + " && python3 setup.py sdist"
    out = os.system(command)
    # print(out)
    if args.repository != '':
        shutil.rmtree(os.path.join(input_dir))
    if errors > 0:
        print("compilation done with ", errors, " errors")
        if len(packages_no_added) > 0:
            print("modules no added in package:\n", packages_no_added)
    final_file = os.path.join(output_dir, package_name + "-" + package_version + ".tar.gz")
    shutil.copyfile(
        os.path.join(output_dir, package_name, "dist", package_name + "-" + package_version + ".tar.gz"),
        final_file
    )
    shutil.rmtree(os.path.join(output_dir, package_name))
    print(colored("package dist save in:\n", "green"), final_file)
    print(colored("install with:\n", "green"), colored("pip3 install", "yellow"), final_file)


if __name__ == "__main__":
    main()



import os
from protobuf_compiler.input_folder import define_input


def args_validation(
        git_repository='',
        proto_input_dir='.',
        proto_output_dir='.',
        proto_package_version='',
        git_repository_token=''
):
    # repository defined
    if git_repository != '':
        if not str(git_repository).startswith("https://") and not str(git_repository).startswith("http://"):
            raise ValueError("-g , --git  : copy clone with https. example: https://gitlab.com/netsaj/test-repo.git")
        if str(git_repository).find("gitlab") == -1 and str(git_repository).find("github") == -1:
            raise ValueError("-g , --git  : git repository support for gitlab and github")

    if proto_input_dir == proto_output_dir:
        raise ValueError("src(-d) proto path need be different to output(-o) path ")

    if proto_package_version == '':
        raise ValueError("package name can't empty")

    if git_repository == '':
        input = define_input(proto_input_dir, git_repository, git_repository_token)
        contain_proto = False
        for file in get_list_of_files(input):
            if file.lower().endswith(".proto"):
                contain_proto = True
                break
        if not contain_proto:
            raise ValueError("src(-d) proto path not contain a proto files (*.proto)")


def get_list_of_files(dirName):
    # create a list of file and sub directories
    # names in the given directory
    listOfFile = os.listdir(dirName)
    allFiles = list()
    # Iterate over all the entries
    for entry in listOfFile:
        # Create full path
        fullPath = os.path.join(dirName, entry)
        # If entry is a directory then get the list of files in this directory
        if os.path.isdir(fullPath):
            allFiles = allFiles + get_list_of_files(fullPath)
        else:
            allFiles.append(fullPath)

    return allFiles



import sys, os
import shutil
from colorama import init
from termcolor import colored


def define_input(proto_input_dir,git_repository_url,git_repository_token):
    """
    :param args: command line args
    :type args: argparse.Namespace
    :return: route of the inputdir
    :type: str
    """
    input_dir = proto_input_dir

    # cloning repository

    if git_repository_url != '':
        command = ''
        if str(git_repository_url).lower().find("gitlab") != -1:
            command = "git clone https://oauth2:" + git_repository_token + "@" + str(git_repository_url).replace("https://",
                                                                                                    "").replace(
                "http://", "")
        elif str(git_repository_url).lower().find("github") != -1:
            command = "git clone https://" + git_repository_token + "@" + str(git_repository_url).replace("https://", "").replace(
                "http://", "")
        if (command == ''):
            raise ValueError('only support gitlab and github repositories')
        print(command)
        out = os.system(command + " proto_temp")
        # change input dir for clone dir
        input_dir = os.path.join(os.getcwd(), "proto_temp")
    else:
        input_dir = os.popen("pwd").read().rstrip("\n\r")
        if str(proto_input_dir).startswith("./"):
            input_dir = os.path.join(input_dir, str(proto_input_dir).replace("./", ""))
        elif str(proto_input_dir) == '.':
            raise ValueError('-d , --dir   args not target to root folder, locate proto \
files in a subfolder. example: ./myProtos')
        else:
            input_dir = proto_input_dir
    return str(input_dir)



import pathlib
from setuptools import setup
from protobuf_compiler.main import __version__
# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

setup(
    name='protobuf_compiler',
    version=__version__,
    packages=['protobuf_compiler'],
    url='https://github.com/netsaj/python-protobuf-compiler',
    license='MIT',
    long_description=README,
    long_description_content_type='text/markdown',
    author='Fabio Moreno',
    author_email='fabiomoreno@outlook.com',
    description='compile all protobuf files and create a single package distribution for can be installed with pip',
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
    ],
    install_requires=[
        'grpcio==1.18.0',
        'grpcio-tools==1.18.0',
        'colorama==0.3.3',
        'termcolor==1.1.0',
        'tqdm==4.31.1'
    ],
    entry_points={
        'console_scripts': [
            'protopy = protobuf_compiler.main:main',
            'protobuf-compiler = protobuf_compiler.main:main',
        ],
    },
    scripts=['protopy.py', 'protobuf-compiler.py']
)



import sys, os
import shutil
from colorama import init
from termcolor import colored


def preparing_output(proto_output_dir, proto_package_name):
    output_dir = os.popen("pwd").read().rstrip("\n\r")
    if str(proto_output_dir).startswith("./"):
        output_dir = os.path.join(output_dir, str(proto_output_dir).replace("./", ""))
    elif str(proto_output_dir) != '.':
        output_dir = proto_output_dir
    if not os.path.exists(output_dir):
        os.mkdir(output_dir)
    if os.path.exists(os.path.join(output_dir, proto_package_name)):
        shutil.rmtree(os.path.join(output_dir, proto_package_name))
    os.mkdir(os.path.join(output_dir, proto_package_name))
    asd = os.path.join(output_dir, proto_package_name, "__init__.py")
    os.system("touch " + asd)
    return str(output_dir)
